import type { DateRange } from '../dateRange'
import type { EntryTagCollection } from '../entryTag'

/**
 * Defines entries that are already fetched or are embedded directly in the list.
 * Stored in key-value format where the key is the id of the entry,
 * and the value is the entry config itself
 * (defined as `T` based on the type of the entries in the implemented list)
 */
export type ListEntries<T> = { [key: string]: T }

/**
 * Defines entries that are fetched from remote config files.
 * Stored in key-value format where the key is the id of the entry,
 * and the value is the url to the config file.
 */
export type ListRemoteEntries = { [key: string]: string }

/**
 * Defines a list-type template that has config entries defined by id.
 * One of two must be defined: `entries` or `embeddedEntries`.
 * Both can be defined and used simultaneously.
 * Note that in the case of an `id` collision
 * (i.e. both `entries` and `embeddedEntries` have an entry with the same id),
 * the implemented behavior should pick the `embeddedEntry`
 */
export type ListWithEntries<T> = {
  /**
   * Entries that will be fetched from remote config files
   */
  entries?: ListRemoteEntries
  /**
   * Entries that are embedded directly in the list config
   */
  embeddedEntries?: ListEntries<T>
}

/**
 * The resolved instance of a list, which will flatten
 */
export type ResolvedListEntries<T> = ListEntries<Promise<T>>

/**
 * Defines a list that supports tagging the entries
 */
export type ListWithTags<T> = {
  /**
   * the tags to use for filtering entries
   */
  tags?: EntryTagCollection
  /**
   * whether or not tag filtering removes entries completely from view;
   * if false, they will apply a class selector instead
   * in order to manually style (CSS filtering/opacity/etc.)
   */
  removeFromView?: boolean
} & ListWithEntries<T>

/**
 * Defines a list that has warnings on its entries
 */
export type ListWithWarnings<T extends MediaEntry> = {
  /**
   * whether or not to hide the warnings toggle from the list page
   */
  removeHideWarningsToggle?: boolean
} & ListWithEntries<T>

/**
 * Defines an entry with common media-related fields
 */
export type MediaEntry = {
  /**
   * Specifies the date of the entry
   */
  date?: DateRange | string | number
  /**[Supports Markdown]
   * Information to summarize an entry
   */
  description?: string
  /**
   * Tags that correspond to filters on the list page if defined
   */
  tags?: string[]
  /**[Supports Markdown]
   * The title of the entry
   */
  title: string
  /**
   * Information regarding the thumbnail
   */
  thumbnail?: {
    /**
     * Sets the inline-styles for the thumbnail
     */
    style: CSSStyleDeclaration
  }
  /**
   * any content warnings that apply to the entry,
   * which will be used to apply the `.warning` classname
   * to the DOM of a tile
   * and will be displayed with the tile
   */
  warning?: string
}
